import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

public class Checkbook {

    Payee owner;
    double balance;
    List<Transaction> checkbook;

    public Checkbook(double cashAmount) {
        this.balance = cashAmount;
        this.checkbook = new ArrayList<>();
    }

    public Checkbook(Payee owner, double cashAmount) {
        this.owner = owner;
        this.balance = cashAmount;
        this.checkbook = new ArrayList<>();
    }

    public int size() {
        return checkbook.size();
    }

    public boolean isEmpty() {
        return size() == 0;
    }

    public boolean contains(Transaction t){
        return checkbook.contains(t);
    }

    public boolean add(Transaction t){
        boolean result = checkbook.add(t);
        this.balance = checkbookValue();
        return result;
    }

    public boolean remove(Transaction t){
        boolean result = checkbook.remove(t);
        this.balance = checkbookValue();
        return result;
    }

    public Iterator<Transaction> iterator(){
        return checkbook.iterator();
    }

    public Transaction[] toArray() {
        return toArray(checkbook);
    }

    public static Transaction[] toArray(List<Transaction> transactions) {
        Transaction[] array = new Transaction[transactions.size()];
        int index = 0;

        for(Transaction t: transactions) {
            array[index] = t;
            index++;
        }
        return array;
    }

    public double checkbookValue() {
        double value = this.balance;
        for(Transaction t: checkbook) {
            if(t.getType().equals(TransactionType.CREDIT))
                value -= t.getAmount();
            if(t.getType().equals(TransactionType.DEBIT))
                value += t.getAmount();
        }
        return value;
    }

    public Transaction[] getTransactionsForDate(LocalDate aDate) {
        ArrayList<Transaction> result = new ArrayList<>();
        for(Transaction t: checkbook) {
            if(t.getDate().equals(aDate))
                result.add(t);
        }
        return toArray(result);
    }

    public Transaction[] getTransactionsForPayee(Payee payee) {
        ArrayList<Transaction> result = new ArrayList<>();
        for(Transaction t: checkbook) {
            if(t.getPayee().equals(payee))
                result.add(t);
        }
        return toArray(result);
    }

    public Transaction[] getAllDebitTransactions() {
        ArrayList<Transaction> result = new ArrayList<>();
        for(Transaction t: checkbook) {
            if(t.getType() == TransactionType.DEBIT)
                result.add(t);
        }
        return toArray(result);
    }

    public Transaction[] getAllCreditTransactions() {
        ArrayList<Transaction> result = new ArrayList<>();
        for(Transaction t: checkbook) {
            if(t.getType() == TransactionType.CREDIT)
                result.add(t);
        }
        return toArray(result);
    }

    public boolean containsAll(Collection c){
        for(Object o : c) {
            if(!contains((Transaction) o))
                return false;
        }
        return true;
    }

    public boolean addAll(Collection c){
        for(Object o : c) {
            boolean result = add((Transaction) o);
            if(!result) {
                return false;
            }
        }
        return true;
    }

    public boolean removeAll(Collection c){
        for(Object o : c) {
            boolean result = remove((Transaction) o);
            if(!result) {
                return false;
            }
        }
        return true;
    }

    public void clear() {
        checkbook.clear();
    }

    public double getBalance() {
        return balance;
    }
}
