import java.lang.reflect.Array;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

public class GenericAccount<T extends AccountBook> {

    Payee owner;
    double balance;
    List<T> accountRecord;
    Class<T> type;

    public GenericAccount(double cashAmount, Class<T> type) {
        this.balance = cashAmount;
        this.accountRecord = new ArrayList<>();
        this.type = type;
    }

    public GenericAccount(Payee owner, double cashAmount, Class<T> type) {
        this.owner = owner;
        this.balance = cashAmount;
        this.accountRecord = new ArrayList<>();
        this.type = type;
    }

    public Class<T> getClassType() {
        return type;
    }

    public int size() {
        return accountRecord.size();
    }

    public boolean isEmpty() {
        return size() == 0;
    }

    public boolean contains(T t){
        return accountRecord.contains(t);
    }

    public boolean add(T t){
        boolean result = accountRecord.add(t);
        this.balance = accountRecordValue();
        return result;
    }

    public boolean remove(T t){
        boolean result = accountRecord.remove(t);
        this.balance = accountRecordValue();
        return result;
    }

    public Iterator<T> iterator(){
        return accountRecord.iterator();
    }

    public T[] toArray() {
        T[] array = (T[]) Array.newInstance(type, accountRecord.size());
        int index = 0;

        for(T t: accountRecord) {
            array[index] = t;
            index++;
        }
        return array;
    }

    public T[] toArray(List<T> ts, Class<T> c) {
        T[] array = (T[]) Array.newInstance(c, ts.size());
        int index = 0;

        for(T t: ts) {
            array[index] = t;
            index++;
        }
        return array;
    }

    public double accountRecordValue() {
        double value = this.balance;
        for(T t: accountRecord) {
            if(t.getType().equals(TransactionType.CREDIT))
                value -= t.getAmount();
            if(t.getType().equals(TransactionType.DEBIT))
                value += t.getAmount();
        }
        return value;
    }

    public T[] getTransactionsForDate(LocalDate aDate) {
        ArrayList<T> result = new ArrayList<>();
        for(T t: accountRecord) {
            if(t.getDate().equals(aDate))
                result.add(t);
        }

        return toArray(result, type);
    }

    public T[] getTransactionsForPayee(Payee payee) {
        ArrayList<T> result = new ArrayList<>();
        for(T t: accountRecord) {
            if(t.getPayee().equals(payee))
                result.add(t);
        }
        return toArray(result, type);
    }

    public T[] getAllDebitTransactions() {
        ArrayList<T> result = new ArrayList<>();
        for(T t: accountRecord) {
            if(t.getType() == TransactionType.DEBIT)
                result.add(t);
        }
        return toArray(result, type);
    }

    public T[] getAllCreditTransactions() {
        ArrayList<T> result = new ArrayList<>();
        for(T t: accountRecord) {
            if(t.getType() == TransactionType.CREDIT)
                result.add(t);
        }
        return toArray(result, type);
    }

    public boolean containsAll(Collection c){
        for(Object o : c) {
            if(!contains((T) o))
                return false;
        }
        return true;
    }

    public boolean addAll(Collection c){
        for(Object o : c) {
            boolean result = add((T) o);
            if(!result) {
                return false;
            }
        }
        return true;
    }

    public boolean removeAll(Collection c){
        for(Object o : c) {
            boolean result = remove((T) o);
            if(!result) {
                return false;
            }
        }
        return true;
    }

    public void clear() {
        accountRecord.clear();
    }

    public double getBalance() {
        return balance;
    }

}
