import org.junit.Test;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.concurrent.atomic.AtomicReference;

import static org.junit.Assert.*;

public class CheckbookTest {

    Payee Lauren;
    Payee Dan;
    Payee Hayden;
    Checkbook checkbook;
    Transaction transaction1;
    Transaction transaction2;
    Transaction transaction3;
    Transaction transaction4;
    Transaction transaction5;
    Transaction transaction6;

    @org.junit.Before
    public void setUp() throws Exception {
        Lauren = new Payee("Green", "Lauren", "lauren@email");
        Dan = new Payee("Green", "Dan", "dan@email");
        Hayden = new Payee("Stewart", "Hayden", "hayden@email");
        transaction1 = new Transaction("Pizza", Dan, TransactionType.CREDIT, 15.50);
        transaction2 = new Transaction("Cheesesteaks", Dan, TransactionType.CREDIT, 22.75);
        transaction3 = new Transaction("Hot Dogs", Dan, TransactionType.DEBIT, 5.25);
        transaction4 = new Transaction("Phillies Tickets", Hayden, TransactionType.CREDIT, 45.00);
        transaction5 = new Transaction("Eagles Tickets", Hayden, TransactionType.DEBIT, 150.00);
        transaction6 = new Transaction("Movie Tickets", Lauren, TransactionType.DEBIT, 22.99);
        checkbook = new Checkbook(Lauren, 5000);
    }

    @Test
    public void testConstructor() {
        Checkbook checkbook2 = new Checkbook(500);
        double expected = 500;
        double actual = checkbook2.getBalance();
        assertEquals(expected, actual);
    }

    @Test
    public void testAddandContains1() {
        checkbook.add(transaction1);
        assertTrue(checkbook.contains(transaction1));
    }

    @Test
    public void testAddandContains2() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        assertTrue(checkbook.contains(transaction2));
    }

    @Test
    public void testAddandContains3() {
        checkbook.add(transaction3);
        checkbook.add(transaction1);
        assertTrue(checkbook.contains(transaction1));
    }

    @Test
    public void testContainsFalse() {
        checkbook.add(transaction3);
        checkbook.add(transaction1);
        assertFalse(checkbook.contains(transaction2));
    }

    @Test
    public void testIsEmpty() {
        assertTrue(checkbook.isEmpty());
    }

    @Test
    public void testIsEmptyAndRemove() {
        checkbook.add(transaction3);
        checkbook.remove(transaction3);
        assertTrue(checkbook.isEmpty());
    }

    @Test
    public void testIsEmptyAndClear() {
        checkbook.add(transaction3);
        checkbook.clear();
        assertTrue(checkbook.isEmpty());
    }

    @Test
    public void testIsEmptyFalse() {
        checkbook.add(transaction3);
        assertFalse(checkbook.isEmpty());
    }

    @Test
    public void testToArray() {
        checkbook.add(transaction3);
        Transaction[] expected = {transaction3};
        Transaction[] actual = checkbook.toArray();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testToArray2() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        Transaction[] expected = {transaction3, transaction5};
        Transaction[] actual = checkbook.toArray();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testToArray3() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction3, transaction5, transaction6};
        Transaction[] actual = checkbook.toArray();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testIterator() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Iterator<Transaction> it = checkbook.iterator();
        Transaction expected = transaction3;
        Transaction actual = it.next();

        assertEquals(expected, actual);
    }

    @Test
    public void testIterator2() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Iterator<Transaction> it = checkbook.iterator();
        Transaction expected = transaction5;
        it.next();
        Transaction actual = it.next();

        assertEquals(expected, actual);
    }

    @Test
    public void testIterator3() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Iterator<Transaction> it = checkbook.iterator();
        Transaction expected = transaction6;
        it.next();
        it.next();
        Transaction actual = it.next();

        assertEquals(expected, actual);
    }

    @Test
    public void testForDate() {
        checkbook.add(transaction3);
        Transaction[] expected = {transaction3};
        Transaction[] actual = checkbook.getTransactionsForDate(LocalDate.now());

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testForDate2() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        Transaction[] expected = {transaction3, transaction5};
        Transaction[] actual = checkbook.getTransactionsForDate(LocalDate.now());

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testForDate3() {
        checkbook.add(transaction3);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction3, transaction5, transaction6};
        Transaction[] actual = checkbook.getTransactionsForDate(LocalDate.now());

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testForPayee1() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction1, transaction2, transaction3};
        Transaction[] actual = checkbook.getTransactionsForPayee(Dan);

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testForPayee2() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction4, transaction5};
        Transaction[] actual = checkbook.getTransactionsForPayee(Hayden);

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testForPayee3() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction6};
        Transaction[] actual = checkbook.getTransactionsForPayee(Lauren);

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testAllCredits() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction1, transaction2, transaction4};
        Transaction[] actual = checkbook.getAllCreditTransactions();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testAllCredits2() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        Transaction[] expected = {transaction1, transaction2};
        Transaction[] actual = checkbook.getAllCreditTransactions();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testAllCredits3() {
        checkbook.add(transaction1);
        Transaction[] expected = {transaction1};
        Transaction[] actual = checkbook.getAllCreditTransactions();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testAllDebits() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        Transaction[] expected = {transaction3, transaction5, transaction6};
        Transaction[] actual = checkbook.getAllDebitTransactions();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testAllDebits2() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        Transaction[] expected = {transaction3};
        Transaction[] actual = checkbook.getAllDebitTransactions();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testAllDebits3() {
        checkbook.add(transaction3);
        Transaction[] expected = {transaction3};
        Transaction[] actual = checkbook.getAllDebitTransactions();

        assertTrue(Arrays.equals(expected, actual));
    }

    @Test
    public void testContainsAll() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);
        testArray.add(transaction5);

        assertTrue(checkbook.containsAll(testArray));
    }

    @Test
    public void testContainsAll2() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);
        testArray.add(transaction5);

        assertFalse(checkbook.containsAll(testArray));
    }

    @Test
    public void testContainsAll3() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);

        assertTrue(checkbook.containsAll(testArray));
    }

    @Test
    public void testAddAll() {
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);
        testArray.add(transaction5);
        checkbook.addAll(testArray);

        assertTrue(checkbook.containsAll(testArray));
    }

    @Test
    public void testAddAll2() {
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);
        testArray.add(transaction5);
        testArray.add(transaction2);
        checkbook.addAll(testArray);

        assertTrue(checkbook.containsAll(testArray));
    }

    @Test
    public void testAddAll3() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction4);
        testArray.add(transaction5);
        testArray.add(transaction6);
        checkbook.addAll(testArray);

        assertTrue(checkbook.containsAll(testArray));
    }

    @Test
    public void testRemoveAll() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        checkbook.add(transaction4);
        checkbook.add(transaction5);
        checkbook.add(transaction6);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);
        testArray.add(transaction5);
        checkbook.removeAll(testArray);

        assertFalse(checkbook.containsAll(testArray));
    }

    @Test
    public void testRemoveAll2() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction3);
        testArray.add(transaction2);
        checkbook.removeAll(testArray);

        assertTrue(checkbook.isEmpty());
    }

    @Test
    public void testRemoveAll3() {
        checkbook.add(transaction1);
        checkbook.add(transaction2);
        checkbook.add(transaction3);
        ArrayList<Transaction> testArray = new ArrayList<>();
        testArray.add(transaction1);
        testArray.add(transaction2);
        checkbook.removeAll(testArray);

        assertFalse(checkbook.containsAll(testArray));
    }

    @org.junit.After
    public void tearDown() throws Exception {
    }
}